// area of interest
// map client area is limited to this region
var wms_minx = -110.1387; 	// degrees, full-state minx
var wms_maxx = -101.8087; 	// degrees, full-state boundary maxx
var wms_miny = 30.7; 		// degrees, full-state boundary miny
var wms_maxy = 37.55; 		// degrees, full-state boundary maxy

// define array of blocksizes (zoom levels). Zoom levels are in SRS units
// limited to exactly 10 in the client html and functions javascript
var blockSizes = [
	0.0015625,
	0.003125,
	0.00625,
	0.0125, 
	0.025, 
	0.05,
	0.1,
	0.4,
	0.8,
	2.4];

// Define Layers to be displayed
// Provide the layer parameters in the following order:
//		shortName, 			// short name of the layer
//		label, 				// label assigned to the layer in the interface
//		server, 			// root URI for the layer
//		baseParameters, 	// base query parameters (in addition to WMS specific ones)
//		version, 			// wms server version
//		format, 			// returned file format
//		style, 				// layer style specification
//		layers, 			// service layers
//		transparent,		// WMS transparency - i.e. whether or not the background should be rendered transparent
//		opacity				// Client layer opacity 0=transparent, 1=completely opaque. May be specified as a real no.

var Layers = new Array(
	new Layer(
		"doqq05",
		"2005 New Mexico DOQQs",
		"http://edacdata2.unm.edu/cgi-bin/mapserv",
		"map=doqq05/doqq05_geogcs.map",
		"1.1.1",
		"image/jpeg",
		"",
		"modis_nm,landsat,doqq05_scale_limited,doqq06_gaps_scale_limited",
		"TRUE",
		"1.0"),
	new Layer(
		"usgsRoads",
		"USGS Major Roads",
		"http://catalog4.usgs.umr.edu/cgi-bin/basevector",
		"",
		"1.1.1",
		"image/png",
		"",
		"usatlasstates,usatlasroads",
		"TRUE",
		"1.0"),
	new Layer(
		"radar",
		"NEXRAD Radar",
		"http://mesonet.agron.iastate.edu/cgi-bin/mapserv/mapserv.radar",
		"map=/mesonet/www/html/GIS/apps/wms/compradar.map",
		"1.1.1",
		"image/png",
		"",
		"nexrad",
		"TRUE",
		"0.65")
)

// Define the location map object
// Use the same parameter order as used for map layers defined above
var locationMap = new Layer(
	"usgsRoads",
	"USGS Major Roads",
	"http://catalog4.usgs.umr.edu/cgi-bin/basevector",
	"",
	"1.1.1",
	"image/png",
	"",
	"usatlasstates,usatlasroads",
	"TRUE")

// Additional Client Variables
// Define the width and height of individual tiles requested from WMS servers
var tileWidth = 200; // pixels
var tileHeight = 200; // pixels

// Dimensions of the viewable map area - the size of the map displayed in the client
var viewableMapWidth = 800; // pixels
var viewableMapHeight = 600; // pixels

// Width of the location (reference) map. The height is automatically calculated from the map width and height
var locationMapWidth = 125; //pixels


//////////////////////////// Don't modify values below here /////////////////////////
// global vars
var wholeMapTop = 0;
var wholeMapLeft = 0;

var iAmDragging = false;
var dragStartX = 0;
var dragStartY = 0;

var rowMode = 'locked';
var columnMode = 'locked';


var wms_aspectRatio = (wms_maxx - wms_minx) / (wms_maxy - wms_miny); // width/height

var locationMapHeight = locationMapWidth / wms_aspectRatio; //pixels


// initialize variables that vary by scale
var wms_blocksize = blockSizes[9]; // degrees
var wholeMapColumnCount = Math.ceil((wms_maxx - wms_minx)/wms_blocksize);
var wholeMapRowCount = Math.ceil((wms_maxy - wms_miny)/wms_blocksize);
var wholeMapWidth = wholeMapColumnCount * tileWidth; // pixels
var wholeMapHeight = wholeMapRowCount * tileHeight; // pixels
var mapCenterX = Math.round(wholeMapWidth/2); // pixels
var mapCenterY = Math.round(wholeMapHeight/2); // pixels
var locationTileWidth = wholeMapColumnCount * (locationMapWidth / wholeMapWidth); // pixels
var locationTileHeight = wholeMapRowCount * (locationMapHeight / wholeMapHeight); // pixels
var locationPixelScaleX = locationMapWidth / wholeMapWidth; // ratio
var locationPixelScaleY = locationMapHeight / wholeMapHeight; // ratio
var locationWindowWidth = locationMapWidth/(wholeMapWidth / viewableMapWidth); // location map pixels
var locationWindowHeight = locationMapHeight/(wholeMapHeight / viewableMapHeight); // location map pixels


// Initialize the map center values
var mapCenterStartX = Math.round(wholeMapWidth);
var mapCenterStartY = Math.round(wholeMapHeight);

