// layer object constructor function and associated prototype assignments
function Layer(shortName,label,server,baseParameters,version,format,style,layers,transparent,opacity) {
	this.shortName = shortName; // short name of the layer
	this.label = label; // label assigned to the layer in the interface
	this.server = server; // root URI for the layer
	this.baseParameters = baseParameters; //base query parameters (in addition to WMS specific ones)
	this.version = version; // wms server version
	this.format = format; // returned file format
	this.style = style; // layer style specification
	this.layers = layers; // service layers
	this.transparent = transparent;
	this.opacity = opacity;
}

Layer.prototype.wmsRequest = function( ) { 
	var wmsString = this.server;
	if (this.baseParameters != "") {
		wmsString = wmsString + "?" + this.baseParameters + "&WMTVER=" + this.version + "&SERVICE=WMS&REQUEST=GetMap&SRS=EPSG:4326&FORMAT=" + this.format + "&STYLES=" + this.style + "&LAYERS=" + this.layers + "&TRANSPARENT=" + this.transparent + "&WIDTH=" + tileWidth + "&HEIGHT=" + tileHeight + "&";
	}
	else {
		wmsString = wmsString + "?WMTVER=" + this.version + "&SERVICE=WMS&REQUEST=GetMap&SRS=EPSG:4326&FORMAT=" + this.format + "&STYLES=" + this.style + "&LAYERS=" + this.layers + "&TRANSPARENT=" + this.transparent + "&WIDTH=" + tileWidth + "&HEIGHT=" + tileHeight + "&";
	}
	return  wmsString;
}



function pageInit() {
   setWholeMapSize((wholeMapColumnCount * tileWidth), (wholeMapRowCount * tileHeight)); //ok
   setViewableMapSize(viewableMapWidth, viewableMapHeight); //ok

   var wholeMap = $( 'wholeMap' );
   var locationMap = $( 'locationMap' );
   locationMap.style.width = locationMapWidth + "px";
   locationMap.style.height = locationMapHeight + "px";
   $( 'zoomLevel_10').style.backgroundColor = 'red';
   zoomTo(mapCenterX,mapCenterY);
   getLocationMap(locationMap); //ok
   Event.observe('viewableMap', 'mousedown', startDrag);
   Event.observe('viewableMap', 'mousemove', dragging);
   Event.observe('viewableMap', 'mouseup', stopDrag);
   Event.observe($('zoomLevel_01'), 'click', zoomTo_01);
   Event.observe($('zoomLevel_02'), 'click', zoomTo_02);
   Event.observe($('zoomLevel_03'), 'click', zoomTo_03);
   Event.observe($('zoomLevel_04'), 'click', zoomTo_04);
   Event.observe($('zoomLevel_05'), 'click', zoomTo_05);
   Event.observe($('zoomLevel_06'), 'click', zoomTo_06);
   Event.observe($('zoomLevel_07'), 'click', zoomTo_07);
   Event.observe($('zoomLevel_08'), 'click', zoomTo_08);
   Event.observe($('zoomLevel_09'), 'click', zoomTo_09);
   Event.observe($('zoomLevel_10'), 'click', zoomTo_10);
   Event.observe($('refreshImage'), 'click', refreshData)
}

function calcScale(y1, x1, y2, x2) {
	var pi = 3.141593;
	var lat_1 = y1 * pi / 180;
	var lon_1 = x1 * pi / 180;
	var lat_2 = y2 * pi / 180;
	var lon_2 = x2 * pi / 180;
	var rad = 6372.795;
	var result = rad * Math.atan(Math.sqrt(Math.pow((Math.cos(lat_2) * Math.sin(lon_1 - lon_2)),2) + Math.pow(( (Math.cos(lat_1) * Math.sin(lat_2) - Math.sin(lat_1) * Math.cos(lat_2) * Math.cos(lon_1 - lon_2))),2)) / ((Math.sin(lat_1) * Math.sin(lat_2)) + (Math.cos(lat_1) * Math.cos(lat_2) * Math.cos(lon_1 - lon_2))));
	return result
}

function clearTiles() {
var wholeMap = $( 'wholeMap' );
var allTiles = wholeMap.getElementsByTagName('img');
for(i = 0; i < allTiles.length; i++) {
	wholeMap.removeChild(allTiles[i]);
	i-- //we're removing a node; make sure that we don't skip the next one
	}
}

function refreshData(event) {
	clearTiles();
	//alert("The tiles have now been cleared")
	calcVisibleTiles();
}

function divmsg(event) {
	alert("Did it!");
}

function highlight(event) {this.style.border = 'thin solid yellow';}
function lowlight(event) {this.style.border = 'none';}

function zoomTo_01(event) {
	resetZoomBtns();
	$( 'zoomLevel_01').style.backgroundColor = 'red';
	rescale(0);
}

function zoomTo_02(event) {
	resetZoomBtns();
	$( 'zoomLevel_02').style.backgroundColor = 'red';
	rescale(1);
}

function zoomTo_03(event) {
	resetZoomBtns();
	$( 'zoomLevel_03').style.backgroundColor = 'red';
	rescale(2);
}

function zoomTo_04(event) {
	resetZoomBtns();
	$( 'zoomLevel_04').style.backgroundColor = 'red';
	rescale(3);
}

function zoomTo_05(event) {
	resetZoomBtns();
	$( 'zoomLevel_05').style.backgroundColor = 'red';
	rescale(4);
}

function zoomTo_06(event) {
	resetZoomBtns();
	$( 'zoomLevel_06').style.backgroundColor = 'red';
	rescale(5);
}

function zoomTo_07(event) {
	resetZoomBtns();
	$( 'zoomLevel_07').style.backgroundColor = 'red';
	rescale(6);
}

function zoomTo_08(event) {
	resetZoomBtns();
	$( 'zoomLevel_08').style.backgroundColor = 'red';
	rescale(7);
}

function zoomTo_09(event) {
	resetZoomBtns();
	$( 'zoomLevel_09').style.backgroundColor = 'red';
	rescale(8);
}

function zoomTo_10(event) {
	resetZoomBtns();
	$( 'zoomLevel_10').style.backgroundColor = 'red';
	rescale(9);
}

function resetZoomBtns() {
	$( 'zoomLevel_01' ).style.backgroundColor = 'transparent';
	$( 'zoomLevel_02' ).style.backgroundColor = 'transparent';
	$( 'zoomLevel_03' ).style.backgroundColor = 'transparent';
	$( 'zoomLevel_04' ).style.backgroundColor = 'transparent';
	$( 'zoomLevel_05' ).style.backgroundColor = 'transparent';
	$( 'zoomLevel_06' ).style.backgroundColor = 'transparent';
	$( 'zoomLevel_07' ).style.backgroundColor = 'transparent';
	$( 'zoomLevel_08' ).style.backgroundColor = 'transparent';
	$( 'zoomLevel_09' ).style.backgroundColor = 'transparent';
	$( 'zoomLevel_10' ).style.backgroundColor = 'transparent';
}

function calcCenter() {
	//messages.innerHTML = wholeMap.style.left + "(" + crosshairLeft + "), " + wholeMap.style.top + "(" + crosshairTop + ")";
}

function rescale(level) {
	var previousWholeMapWidth = wholeMapWidth;
	var previousWholeMapHeight = wholeMapHeight;
	var previousBlocksize = wms_blocksize; // degrees
	var wholeMap = $( 'wholeMap' );
	wms_blocksize = blockSizes[level]; // degrees
	wholeMapColumnCount = Math.ceil((wms_maxx - wms_minx)/wms_blocksize)
	wholeMapRowCount = Math.ceil((wms_maxy - wms_miny)/wms_blocksize)
	wholeMapWidth = wholeMapColumnCount * tileWidth; // pixels
	wholeMapHeight = wholeMapRowCount * tileHeight; // pixels
	mapCenterX = Math.round(mapCenterX * (wholeMapWidth/previousWholeMapWidth));
	mapCenterY = Math.round(mapCenterY * (wholeMapHeight/previousWholeMapHeight));
	locationTileWidth = locationMapWidth / wholeMapColumnCount ; //pixels
	locationTileHeight = locationMapHeight / wholeMapRowCount ; //pixels	
	locationPixelScaleX = locationMapWidth / wholeMapWidth; // ratio
	locationPixelScaleY = locationMapHeight / wholeMapHeight; // ratio
	locationWindowWidth = locationMapWidth/(wholeMapWidth / viewableMapWidth); // location map pixels
	locationWindowHeight = locationMapHeight/(wholeMapHeight / viewableMapHeight); // location map pixels

	var allTiles = wholeMap.getElementsByTagName('img');
	for(i = 0; i < allTiles.length; i++) {
		var id = allTiles[i].getAttribute('id');
		wholeMap.removeChild(allTiles[i]);
		i-- //we're removing a node; make sure that we don't skip the next one
	}	
	zoomTo(mapCenterX,mapCenterY);
}

function zoomTo(centerX,centerY) {
	var wholeMap = $( 'wholeMap' );
	wholeMap.style.left = Math.round(-centerX + (viewableMapWidth/2)) + 'px';
	wholeMap.style.top = Math.round(-centerY + (viewableMapHeight/2)) + 'px';
	
	var crosshairLocation = $( 'crosshair' );
	crosshairTop = Math.round(((- stripPx(wholeMap.style.top) + (viewableMapHeight/2)) * locationPixelScaleY) - 15) + "px";
	crosshairLeft = Math.round(((- stripPx(wholeMap.style.left) + (viewableMapWidth/2)) * locationPixelScaleX) - 15) + "px";
	crosshairLocation.style.top = crosshairTop;
	crosshairLocation.style.left = crosshairLeft;
	
	calcVisibleTiles()
}

function setWholeMapSize(width, height) {
   var wholeMap = $( 'wholeMap' );
   wholeMap.style.width = width + "px";
   wholeMap.style.height = height + "px";
}

function setViewableMapSize(width, height) {
   var viewableMap = $( 'viewableMap' );
   viewableMap.style.width = width + "px";
   viewableMap.style.height = height + "px";
}

function startDrag(event) {
   iAmDragging = true

   //capture the initial point where dragging began
   dragStartX = Event.pointerX(event)
   dragStartY = Event.pointerY(event)

   //capture the current top,left of wholeMap
   var wholeMap = $( 'wholeMap' )
   wholeMapTop = stripPx(wholeMap.style.top)
   wholeMapLeft = stripPx(wholeMap.style.left)
   
   //capture the original map center
   mapCenterStartX = mapCenterX;
   mapCenterStartY = mapCenterY;

   //change the mouse cursor
   wholeMap.style.cursor = '-moz-grab'
   Event.stop(event)
}

function stopDrag(event) {
   iAmDragging = false

   //change the mouse cursor
   var wholeMap = $( 'wholeMap' )
   wholeMap.style.cursor = ''
   Event.stop(event)
}

function dragging(event) {
   if(iAmDragging) {
      var wholeMap = $( 'wholeMap' );
      	wholeMap.style.left = (wholeMapLeft + (Event.pointerX(event) - dragStartX)) + "px";
      	mapCenterX = Math.round(- stripPx(wholeMap.style.left) + (viewableMapWidth/2)); //pixels

      wholeMap.style.top = (wholeMapTop + (Event.pointerY(event) - dragStartY)) + "px";
      mapCenterY = Math.round(- stripPx(wholeMap.style.top) + (viewableMapHeight/2)); //pixels


		var crosshairLocation = $( 'crosshair' );
		crosshairTop = Math.round(((- stripPx(wholeMap.style.top) + (viewableMapHeight/2)) * locationPixelScaleY) - (15)) + "px";
		crosshairLeft = Math.round(((- stripPx(wholeMap.style.left) + (viewableMapWidth/2)) * locationPixelScaleX) - (15)) + "px";
		crosshairLocation.style.top = crosshairTop;
		crosshairLocation.style.left = crosshairLeft;
		var messages = $( 'messages' );
		//messages.innerHTML = wholeMap.style.left + "(" + crosshairLeft + "), " + wholeMap.style.top + "(" + crosshairTop + ") / " + wholeMapWidth;


      calcVisibleTiles()
   }
   Event.stop(event)
}

function stripPx(value) {
   if(value == "") {
      return 0
   }
   else {
      return parseFloat(value.substring(0, value.length - 2))
   }
}

function calcVisibleTiles() {
   var viewableMap = $( 'viewableMap' );
   var columnCount = Math.ceil((stripPx(viewableMap.style.width) / tileWidth));
   var rowCount = Math.ceil((stripPx(viewableMap.style.height) / tileHeight));

   var wholeMap = $( 'wholeMap' );
   var startingColumn = Math.floor(Math.abs(stripPx(wholeMap.style.left) / tileWidth));
   var startingRow = Math.floor(Math.abs(stripPx(wholeMap.style.top) / tileHeight));

   var endingColumn = Math.min((startingColumn + columnCount),wholeMapColumnCount);
   var endingRow = Math.min((startingRow + rowCount),wholeMapRowCount);
   
   var visibleTiles = {};
   for(r = startingRow; r <= endingRow; r++) {
      for(c = startingColumn; c <= endingColumn; c++) {
         var tileId = addTile(wholeMap, c, r);
         visibleTiles[tileId] = true;
      }
   }
}


function addTile(wholeMap, c, r) {
	// loop through the Layers
	var tileId01 = "";
	var bboxCorners = "" + (wms_minx + (c * wms_blocksize)) + "," + (wms_miny + ((wholeMapRowCount - r - 1) * wms_blocksize)) + "," + (wms_minx + ((c + 1) * wms_blocksize)) + "," + (wms_miny + (((wholeMapRowCount - r - 1) + 1) * wms_blocksize));
	var wms_bbox = "bbox=" + bboxCorners;
	var curScale = calcScale((wms_miny + ((wholeMapRowCount - r - 1) * wms_blocksize)),(wms_minx + (c * wms_blocksize)),(wms_miny + ((wholeMapRowCount - r - 1) * wms_blocksize)),(wms_minx + ((c + 1) * wms_blocksize)));
	var miScale = curScale * 0.62137119;
	$('scaleval').update(curScale.toFixed(1) + " km/" + miScale.toFixed(1) + " mi")
	for (var i = 0 ; i < Layers.length ; i++) {
		tileId01 = zeroPad(c) + "-" + zeroPad(r) + Layers[i].shortName;
		var filename_l01 = Layers[i].wmsRequest() + wms_bbox;
		var tileTop = r * tileHeight;
		var tileLeft = c * tileWidth;
		
		//is the img already loaded?
		var img = $( tileId01 );
		if(!img){
			img = document.createElement("img");
			wholeMap.appendChild(img);
			img.style.position = 'absolute';
			img.style.top = tileTop + "px";
			img.style.left = tileLeft + "px";
			img.style.opacity = Layers[i].opacity;
			img.setAttribute("id", tileId01);
			img.src = filename_l01;
	  	}	
	}	
}

function zeroPad(value) {
   if(value < 10) {
      return "00" + value
   }
   else if(value < 100) {
   	return "0" + value
   }
   else {
      return value
   }
}

function getLocationMap(locationmap) {
	var img = document.createElement("img")
	var locationBBOX = "BBOX=" + wms_minx + "," + wms_miny + "," + wms_maxx + "," + wms_maxy;
	img.src = locationMap.wmsRequest() + locationBBOX + "&WIDTH=" + locationMapWidth + "&HEIGHT=" + locationMapHeight;
	img.style.position = 'absolute'
	img.style.top = '0px'
	img.style.left = '0px'
	img.setAttribute("id","locationMapImage")
	locationmap.appendChild(img)
}


Event.observe(window, 'load', pageInit)